<?php

/**
* @property nc_Core $nc_core
*/
class SPExport {
    
    // ID раздела, в котором инициализирован класс
    var $sub;
    // дерево родителей раздела
    var $sub_parent_tree;
    // сайт, к которому относится раздел
    var $catalogue;
    // настройки модуля в разделе(в случае инициализации из раздела) 
    var $settings = array();
    // объект ядра
    var $nc_core = null;
    // местное кеширование компонентов
    var $components = array();
    
    /**
    * соответствие типов социалок их адишникам в классификаторе
    * @todo переписать на получение из базы
    */
    var $id2type = array(
        1=>'FB',
        2=>'VK',
        3=>'OK'
    );
    /**
    * константы условий выгрузки
    */
    const CONDITION_CREATED = 1;  // создание 
    const CONDITION_CHECKED = 2;  // включение
    /**
    * константы режима альбома
    */
    const ALBUM_CREATE = 1;     // пытаться создать альбом, если не указан ID
    const ALBUM_EXISTING = 2;   // выгружать только если ID указан
    // собственные таблицы
    const TABLE_QUEUE = 'SPE_Export_Queue'; 
    const TABLE_QUEUE_FILES = 'SPE_Export_Queue_Files';
    
    /**
    * список тпов соцалок. часто используется -вынес в статику
    */
    static $social = array('VK','FB','OK');
    
    /**
    * путь для аяксового обработчика различных служебных дапросов
    */
    var $ajax_handler = '';
    
    /**
    * подписывается на события добавления и изменения записей компонентов
    */
    public function __construct($sub=0) {
        global $nc_core,$HTTP_HOST,$SUB_FOLDER;
        $this->nc_core = $nc_core;    
        if(empty($sub)){
            // объект класса создается при загрузке нетката - биндим обработчики
            /**
            * @todo: вешаться еще и на изменение значения поля
            */
            $this->nc_core->event->bind($this, 'addMessage');
            $this->nc_core->event->bind($this, 'checkMessage');   
            //$this->nc_core->event->bind($this, 'updateMessage');
        }else{
            $this->_read_settings($sub);
            $this->ajax_handler = 'http://'.$HTTP_HOST.$SUB_FOLDER.'/netcat/modules/social_photo_export/ajax_handler.php'; 
        }   
    }
    
    /**
    * коллбек создания 
    */
    public function addMessage($Catalogue_ID, $Subdivision_ID,$Sub_Class_ID, $Class_ID,$Message_ID){
        $this->_check_need_export(self::CONDITION_CREATED,$Catalogue_ID, $Subdivision_ID,$Sub_Class_ID, $Class_ID,$Message_ID);
    }
    
    /**
    * коллбек включения 
    */
    public function checkMessage($Catalogue_ID, $Subdivision_ID,$Sub_Class_ID, $Class_ID,$Message_ID){
        $this->_check_need_export(self::CONDITION_CHECKED,$Catalogue_ID, $Subdivision_ID,$Sub_Class_ID, $Class_ID,$Message_ID);
    }
    
    /**
    * проверяет, подлежит ли данная запись выгрузке в социалку, и если да - то записывает ее в очередь
    * 
    * @return void
    */
    protected function _check_need_export($action,$Catalogue_ID, $Subdivision_ID,$Sub_Class_ID, $Class_ID,$Message_ID){
        $this->_read_settings($Subdivision_ID);
        // сравниаем произошедшее с событием, по которому нужно выгружать
        if($action !=$this->settings['SPE_EXPORT_CONDITION']){
            return; // а вот ни фига, не совпало.
        }
        // может приходить как одно значение, так и массив. к общему знаменателю все
        if(!is_array($Message_ID)){
            $Message_ID = array($Message_ID);
        }
        foreach($Message_ID as $_item){
            foreach(self::$social as $type){
                /* 
                если хоть в одну социалку надо выгрузить 
                и 
                (
                    для этого раздела выставлено создание альбома 
                    или 
                    есть ID альбома для этой социалки
                )
                - сохраняем запись
                */
                if(
                    $this->settings['SPE_EXPORT_TO'][$type]
                    &&
                    (
                        $this->settings['SPE_ALBUM_MODE'] == self::ALBUM_CREATE
                        ||
                        $this->settings['SPE_'.$type.'_ALBUM_ID']
                    )
                ){
                    $this->add_record_to_queue($Class_ID,$Sub_Class_ID,$_item);  
                    break; // и берем следующую запись   
                }    
            }
        }
            
    }
    
    /**
    * загружает настройки выгрузки для конкретного раздела
    * может использоватеся как бекдор для переинициализации существующего объекта под другой раздел, 
    * если кому жалко памяти под новый:)
    * @param int Subdivision_ID
    * 
    * @return void
    */
    public function _read_settings($sub){
        $this->sub = $sub;
        // настройки из раздела, которые напрямую копируются
        $this->sub_parent_tree = $this->nc_core->subdivision->get_parent_tree($sub);
        // после установки модуля это поле еще нигде не заполенно и приходит пусттая строка, а нужен массив
        if(empty($this->sub_parent_tree[0]['SPE_EXPORT_TO_id'])){
            $this->sub_parent_tree[0]['SPE_EXPORT_TO_id'] = array();
        }
        $fields = array(
            'SPE_FB_ALBUM_ID',
            'SPE_VK_ALBUM_ID',    
            'SPE_OK_ALBUM_ID', 
            'SPE_ALBUM_MODE'=>'SPE_ALBUM_MODE_id',   
            'SPE_EXPORT_CONDITION'=>'SPE_EXPORT_CONDITION_id'  
        );
        foreach($fields as $field=>$key){
            if(is_numeric($field)){
                $field = $key; 
            }    
            $this->settings[$field]= $this->sub_parent_tree[0][$key];
        } 
        // в какие социалки выгружать
        foreach($this->id2type as $id=>$type){
            $this->settings['SPE_EXPORT_TO'][$type]=in_array($id,$this->sub_parent_tree[0]['SPE_EXPORT_TO_id']);    
        }
        // настройки из сайта
        $this->catalogue= $this->nc_core->catalogue->get_by_id($this->sub_parent_tree[0]['Catalogue_ID']);
        foreach(self::$social as $type){
            $need_fields = array('APP_ID','GROUP_ID');
            // одноклассникам нужно больше
            if($type=='OK'){
                $need_fields[] = 'APP_PUBLIC';    
                $need_fields[] = 'APP_PRIVATE';    
            }
            foreach($need_fields as $key){
                $key = 'SPE_'.$type.'_'.$key;
                $this->settings[$key]= $this->catalogue[$key];          
                // если не заполнены все из настроек для конкретной социалки - в нее не выгружаем
                if(empty($this->catalogue[$key])){
                    $this->settings['SPE_EXPORT_TO'][$type] = false; //     
                } 
            }
        }
    }
    
    /**
    * инициализирует выгрузку файлов из поля выводимого инфоблока 
    * @param array переменные окружения сс
    * @param mixed имя или массив имен полей для выгрузки
    * 
    * @return void
    */
    public function export($cc_env,$export_fields){
        // дополняем все записи, подлежащие выгрузке, файлами, которых еще нет в базе 
        $this->_add_files_to_queue($cc_env,$export_fields);
        // получаем список файлов, которые еще не выгружены
        $to_export = $this->_get_files_for_export($cc_env);
        // если нужны одноклассники
        if($to_export['export_to']['OK']){
            $this->_ok_login();
            /*
            // это экспорт сразу в момент вывода списка, но сильно затормаживает. 
            // использовал для отладки, но сейчас дергаю выгрузку в фоне из js
            $this->_ok_export($to_export['records']);
            */
        }
        include('js/export.php');
    }
    
    /**
    * выгружает фото исключительно в OK исключительно по одному инфоблоку
    * используется при фоновом вызове экспорта в одноклассники
    */
    public function ok_export($Sub_Class_ID){
        $to_export = $this->_get_files_for_export(array('Sub_Class_ID'=>$Sub_Class_ID));
        if($to_export['export_to']['OK']){
            $this->_ok_export($to_export['records']);   
        }
    }
    
    /**
    * добавляет запись в очередь на выгрузку
    * @param int ID компонент
    * @param int ID записи
    * 
    * @return mixed ID записи или FALSE
    */
    public function add_record_to_queue($Class_ID,$Sub_Class_ID,$Message_ID){
        if(empty($this->settings)){
            // объект не был проинициализирован под раздел
            return false;
        }
        $query = self::buildInsertQuery(
            self::TABLE_QUEUE,
            array(
                'id'=>'',
                'Class_ID'=>$Class_ID,
                'Sub_Class_ID'=>$Sub_Class_ID,
                'Message_ID'=>$Message_ID,
                'Created'=>self::now_mysql(),
                'to_vk'=>(string)(int)$this->settings['SPE_EXPORT_TO']['VK'],
                'to_fb'=>(string)(int)$this->settings['SPE_EXPORT_TO']['FB'],
                'to_ok'=>(string)(int)$this->settings['SPE_EXPORT_TO']['OK']
            ),
            false,false,true);
        $res = $this->nc_core->db->query($query);
        if($res){
            return $this->nc_core->db->insert_id;
        }
        return false;
    }
    
    /**
    * отдает объект класса компонента, инициализированного по ID, с местным кешированием
    * @param int ID компонента
    * 
    * @return object класса nc_Component
    */
    private function _get_component($Class_ID){
        if(!isset($this->components[$Class_ID])){
            $this->components[$Class_ID] = new nc_Component($Class_ID);  
        }
        return $this->components[$Class_ID];    
    }
    
    /**
    * для подлежащих экспорту записей инфоблока добавляет в базу список файлов
    * @param array переменные окружения сс
    * @param mixed имя или массив имен полей для выгрузки
    * @uses nc_get_arrays_multifield_for_nc_object_list_and_full_php()
    * 
    * @return void
    */
    public function _add_files_to_queue($cc_env,$export_fields){
        if(!is_array($export_fields)){
            $export_fields = array($export_fields);
        }
        // для начала получаем все записи этого инфоблока, подлежащие выгрузке  
        $query = "
            select q.*
            FROM `".self::TABLE_QUEUE."` as q
            WHERE q.Sub_Class_ID = ".$cc_env['Sub_Class_ID']."
        ";
        $records = $this->nc_core->db->get_results($query,ARRAY_A);
        if(empty($records)){
            return; // нечего выгружать
        }
        $files_for_queue = array();
        foreach($records as $record){
            $component = $this->_get_component($record['Class_ID']); 
            $fields = $component->get_fields(0);
            // ищем поле, которое надо выгрузить
            foreach($fields as $field){
                if(in_array($field['name'],$export_fields)){
                    switch($field['type']){
                        case 6:
                            $files_for_queue[] = array(
                                'record_id'=>$record['id'],
                                'path'=> nc_file_path($record['Class_ID'],$record['Message_ID'],$field['name'])
                            );    
                            break;
                        case 11:
                            $res = nc_get_arrays_multifield_for_nc_object_list_and_full_php(array($field),array($record['Message_ID']));
                            if(empty($res)){
                                break;
                            }
                            foreach($res['multifiles'][$record['Message_ID']][$field['name']] as $file){
                                $files_for_queue[] = array(
                                    'record_id'=>$record['id'],
                                    'path'=> $file['Path']
                                );
                            }
                            break;
                            
                    } 
                }
            }
        }
        $query = self::buildInsertQuery(
            self::TABLE_QUEUE_FILES,
            $files_for_queue,
            true,true,true
        );
        $this->nc_core->db->query($query);
    }
    
    /**
    * отдает список файлов указанного окружения, которые еще не экспортировались по назначению 
    * и перечень целевых социалок
    * 
    * @return array()
    */
    private function _get_files_for_export($cc_env){
        $query = "
            select q.*,qf.*
            FROM `".self::TABLE_QUEUE_FILES."` as qf
            JOIN `".self::TABLE_QUEUE."` as q ON qf.record_id = q.id
            WHERE q.Sub_Class_ID = ".$cc_env['Sub_Class_ID']."
                AND
                (
                    (q.to_fb='1' AND qf.fb_id='')
                    OR
                    (q.to_vk='1' AND qf.vk_id='')
                    OR
                    (q.to_ok='1' AND qf.ok_id='')
                )
        ";                        
        $records = $this->nc_core->db->get_results($query,ARRAY_A); 
        if(empty($records)){
            $records = array();
        }
        $ret = array(
            'records'=>$records,
            'export_to'=>array()
        );
        foreach($records as $record){
            foreach(self::$social as $type){
                $ret['export_to'][$type] = $ret['export_to'][$type] || ($record['to_'.mb_strtolower($type)] && empty($record[mb_strtolower($type).'_id']));       
            }
        }
        return $ret;
    }
    
    /**
    * проверяет, что админ группы авторизован или отправляет на авторизацию
    */
    private function _ok_login(){
        global $HTTP_HOST,$SUB_FOLDER;
        $need_fields = array('APP_ID','GROUP_ID','APP_PUBLIC','APP_PRIVATE');
        foreach($need_fields as $key){
            $key = 'SPE_OK_'.$key;
            if(empty($this->catalogue[$key]))
                return; // нет одной из необходимых настроек
            /**
            * @todo: предупреждать об ошибке
            */
        }
        if(empty($_SESSION['SPE_OK_ADMIN'])){
            $ok_set_return_url = $this->ajax_handler;
            $auth_admin_url = $this->get_ok_auth();
            include('js/ok_login.php');
        }    
    }    
    
    /**
    * отдает путь, на который придет пользователь после авторизации на OK
    */
    public function get_ok_redirect($action='ok'){
        global $HTTP_HOST,$SUB_FOLDER;
        return "http://".$HTTP_HOST.$SUB_FOLDER."/netcat/modules/social_photo_export/ok_login_admin.php?sub=".$this->sub;   
    }
    
    /**
    * отдает путь, на который надо отпрвить пользователя для авторизации под OK
    */
    public function get_ok_auth(){
        return "http://www.odnoklassniki.ru/oauth/authorize?client_id=".$this->settings['SPE_OK_APP_ID']."&scope=PHOTO+CONTENT&response_type=code&redirect_uri=".urlencode($this->get_ok_redirect());
    }
    
    /**
    * вытягивает из одноклассников и записывает в сессию инфу о пользовтаеле одноклассников
    */
    function ok_auth($code){
        $user = $this->ok_get_user_by_code($code,$this->get_ok_redirect()); 
        $_SESSION['SPE_OK_ADMIN'] = $user;
        if($_SESSION['SPE_ok_return_to']){
            header('Location: '.$_SESSION['SPE_ok_return_to']); // редиректим туда, откуда авторизация началась  
            unset($_SESSION['SPE_ok_return_to']);
            exit(); 
        }
        header('Location: /netcat/admin/'); // редиректим после авторизации на админку  
        exit();    
    }
    
    /**
    * отдает инфу о пользователе сразу после авторизации, по коду
    */
    private function ok_get_user_by_code($code,$redirect_url){
        $auth = self::curl_exchange('http://api.odnoklassniki.ru/oauth/token.do',
            self::join_params(array(
                'code'=>$code,
                'redirect_uri'=>urlencode($redirect_url),
                'grant_type'=>'authorization_code',
                'client_id'=>$this->settings['SPE_OK_APP_ID'],
                'client_secret'=> $this->settings['SPE_OK_APP_PRIVATE']
            ),'&',false)
        );
        if(empty($auth) || isset($auth->error)){
            return $auth;
        }
        $url = $this->getSignaturedUrl($auth->access_token,array(
            'application_key'=>$this->settings['SPE_OK_APP_PUBLIC'],
            'method'=> 'users.getCurrentUser'
        ));
        $user = self::curl_exchange($url);
        if(is_object($user)){
            $user->access_token = $auth->access_token;
            $user->refresh_token = $auth->refresh_token;
        }
        return $user;    
    }
    
    /**
    * выполняет экспорт фоток в одноклассники
    */
    private function _ok_export($records){
        if(!$this->settings['SPE_OK_ALBUM_ID']
            &&
            (
                !$this->settings['SPE_ALBUM_MODE'] == self::ALBUM_CREATE
                ||
                !$this->settings['SPE_OK_ALBUM_ID'] = $this->_create_ok_album()
            )   
        ){
            return;
        }
        $photos = array(); 
        foreach($records as $key=>$record){  
            if(!$record['to_ok'] || $record['ok_id']){
                // пропускаем уже выгруженые или которые не надо выгружать
                continue;
            }        
            $photos[] = $record['path'];
        }
        //
        /*
        загрузка фоток пачками по 20 штук - пока закомментил,потому что не могу решить проблему установления соответствия 
        исходного файла с его id в одноклассниках
        $offset = 0;
        $max_per_time = 4;
        do{
            $slice = array_slice($photos,$offset,$max_per_time);
            $offset+=$max_per_time;
            $upload_data = $this->get_upload_url(true,sizeof($slice));
            // фаза два
            $pics = array(); 
            foreach($slice as $i=>$path){
                $pics['pic'.($i+1)] = "@".$_SERVER['DOCUMENT_ROOT'].$path;
            }
            echo '<pre>';
            var_dump('$pics'); echo '<br>';
            var_dump($pics); echo '<br>';
            echo '</pre>';
            $upload_res = self::curl_exchange(
                $upload_data->upload_url,
                $pics
            );
            echo '<pre>';
            var_dump('$upload_res'); echo '<br>';
            var_dump($upload_res); echo '<br>';
            echo '</pre>';
            // сохраняем соответствие id и пути
            $match = array();    
            // ФАЗА 3 - сабмит загруженных фото
            $params = array(
                'application_key'=>$this->settings['SPE_OK_APP_PUBLIC'],
                'method'=>'photosV2.commit',
                'photos'=>array() 
            );
            $counter = 0;
            foreach($upload_res->photos as $photo_id=>$photo){
                $match[$photo_id] = $slice[$counter++];
                $params['photos'][] = array(
                    'comment'=>'',
                    'photo_id'=>$photo_id,
                    'token'=>$photo->token
                );
            }
            echo '<pre>';
            var_dump('$match'); echo '<br>';
            var_dump($match); echo '<br>';
            echo '</pre>';
            $params['photos'] = json_encode($params['photos']);
            $url = $this->getSignaturedUrl($_SESSION['SPE_OK_ADMIN']->access_token,$params);
            $ret_data = self::curl_exchange($url);
            echo '<pre>';
            var_dump('$ret_data'); echo '<br>';
            var_dump($ret_data); echo '<br>';
            echo '</pre>';
            foreach($ret_data->photos as $i=>$photo){
                // остается только надеяться,что одноклассники отдают обратно фотки в том-же порядке,что я их туда отправлял
                // проверил - фиг там!!! 
                if($photo->status !='SUCCESS')
                    continue;
                //self::setQueueFileId($slice[$i],'OK',$photo->assigned_photo_id);     
            }
        }while($offset<sizeof($photos));
        */   
        // загрузка по одной штуке
        foreach($photos as $path){
            // фаза 1 - получение урла
            $upload_data = $this->get_upload_url(true); 
            // фаза 2 - аплоад
            $upload_res = self::curl_exchange(
                $upload_data->upload_url,
                array('pic1' => "@".$_SERVER['DOCUMENT_ROOT'].$path)
            );
            // ФАЗА 3 - сабмит загруженных фото
            $upload_res->photos = (array)$upload_res->photos;
            $params = array(
                'application_key'=>$this->settings['SPE_OK_APP_PUBLIC'],
                'comment'=>'',
                'method'=>'photosV2.commit',
                'photo_id'=>array_shift(array_keys($upload_res->photos)),
                'token'=>$upload_res->photos[array_shift(array_keys($upload_res->photos))]->token
            );
            $url = $this->getSignaturedUrl($_SESSION['SPE_OK_ADMIN']->access_token,$params);
            $ret_data = $upload_res = self::curl_exchange($url);
            self::setQueueFileId($path,'OK',$ret_data->photos[0]->assigned_photo_id);  
        }
            
    }            
    
    /**
    * отдает путь для зарузки фото в одноклассники
    * @param bool пробовать обновить токен, если он просрочен
    * @param int количество фоток, под которое запрашивается путь загрузки
    */
    function get_upload_url($try_refresh=true,$count =1){
        $params = array(
            'aid'=>$this->settings['SPE_OK_ALBUM_ID'],
            'application_key'=>$this->settings['SPE_OK_APP_PUBLIC'],
            'count'=>$count,
            'gid'=>$this->settings['SPE_OK_GROUP_ID'],
            'method'=>'photosV2.getUploadUrl'
        );
        $url = $this->getSignaturedUrl($_SESSION['SPE_OK_ADMIN']->access_token,$params);
        $ret_data = self::curl_exchange($url);
        if(!$ret_data){
            exit('cannot get upload url');
        }
        if(strpos($ret_data->error_msg,'PARAM_SESSION_EXPIRED')!==false && $try_refresh){
            $this->ok_refresh_token();
            return $this->get_upload_url(false);
        }
        return $ret_data;
    }
    
    /**
    * создает альбом в одноклассниках для текущего раздела
    */
    function _create_ok_album($try_refresh=true){
        $params = array(
            'method'=>'photos.createAlbum',
            'title'=>$this->sub_parent_tree[0]['Subdivision_Name'],
            'application_key'=>$this->settings['SPE_OK_APP_PUBLIC'],
            'gid'=>$this->settings['SPE_OK_GROUP_ID']     
        );
        $url = $this->getSignaturedUrl($_SESSION['SPE_OK_ADMIN']->access_token,$params);
        $ret_data = self::curl_exchange($url);
        if(strpos($ret_data->error_msg,'PARAM_SESSION_EXPIRED')!==false && $try_refresh){
            $this->_ok_refresh_token();
            return $this->_create_ok_album(false);
        }
        $this->nc_core->db->query("
            update Subdivision 
            set `SPE_OK_ALBUM_ID` = ".mysql_real_escape_string($ret_data)." 
            WHERE Subdivision_ID=".$this->sub." 
                AND `SPE_OK_ALBUM_ID`=''
        ");  
        return $this->settings['SPE_OK_ALBUM_ID'] = $ret_data;    
    }
    
    /**
    * пробует обновить токен для авторизованного в одноклассниках админа
    */
    private function _ok_refresh_token(){
        $params = self::join_params(
            array(
                'refresh_token'=>$_SESSION['SPE_OK_ADMIN']->refresh_token,
                'grant_type'=>'refresh_token',
                'client_id'=>$this->settings['SPE_OK_APP_ID'],
                'client_secret'=>$this->settings['SPE_OK_APP_PRIVATE']
            ),'&',false);
        $auth = self::curl_exchange(
            'http://api.odnoklassniki.ru/oauth/token.do',
            $params
        );
        if($auth->access_token){
            $_SESSION['SPE_OK_ADMIN']->access_token = $auth->access_token;    
        }
    }
    
    /**
    * отдает правильно сформированный подписанный урл для одноклассников 
    */
    private function getSignaturedUrl($access_token,$params){
        ksort($params);
        $params_joined = self::join_params($params,'',false);
        $params_encoded = self::join_params($params);
        return $url = 'http://api.odnoklassniki.ru/fb.do?access_token=' . $access_token 
            .'&'.$params_encoded
            .'&sig=' . md5(
                $params_joined . md5($access_token .$this->settings['SPE_OK_APP_PRIVATE'])
                );    
    }
    
    /**
    * для указанного файла выставляет ID после выгрузки в социалку
    */                      //$path,'VK',$_POST['response'][$key]['id']
    function setQueueFileId($path,$type,$id){
        global $nc_core;
        $query = self::buildUpdateQuery(
            self::TABLE_QUEUE_FILES,
            array(strtolower($type).'_id'=>$id),
            "path = '".$path."'"
        );
        $nc_core->db->query($query);        
    }
    
    /**
    * статические методы - хелперы
    */
        
    /**
    * строит строку запроса для вставки, экранирует строки
    * @param string имя таблицы для вставки
    * @param array ассоциативный массив или массив аналогичных ассоциативных массивов с данными
    * @param bool флаг, что происходит вставка стразу нескольких строк
    * @param bool флаг возвращать данные, не дожидаясь окончания запроса
    * @param bool флаг не вставлять данные, если в таблице дублируется первичный ключ
    * @uses filterDataForQuery
    * 
    * @author Rolland
    * 
    */
    static function buildInsertQuery($table,$data = array(),$is_multiple_insert = false,$delayed=false,$ignore = false){
        $ins_query = "INSERT ".($delayed?' DELAYED ':'').($ignore?' IGNORE ':'')." INTO `".$table."` " ;
        if($is_multiple_insert){
            $ins_query .=" (`".join("`,`",array_keys($data[array_shift(array_keys($data))]))."`)";  // я расчитываю,что $data - массив из ассоциативных массивов с одинаковым набором и порялком ключей, соответствующих полям в таблице для вставки
            $ins_query .=" VALUES ";
            foreach($data as $row_data){
                self::filterDataForQuery($row_data);
                $ins_query .="(".join(',',$row_data)."),";
            }
            $ins_query = rtrim($ins_query,",");
        }else{
            $ins_query .="(`".join("`,`",array_keys($data))."`)";
            $ins_query .=" VALUES ";
            self::filterDataForQuery($data);
            $ins_query .="(".join(',',$data).")";
        }
        return $ins_query;
        
    }
    
        
    /**
    * строит строку запроса для изменения, экранирует строки
    * @param string имя таблицы для вставки
    * @param array ассоциативный массив или массив аналогичных ассоциативных массивов с данными 
    * @param string условия изменения
    * @uses filterDataForQuery
    * 
    * @author Rolland
    * 
    */
    function buildUpdateQuery($table,$data = array(),$where){
        $upd_query = "UPDATE `".$table."` SET " ;
        self::filterDataForQuery($data);
        foreach($data as $field=>$value){
            $upd_query .= " `".$field."`=".$value.",";        
        }
        $upd_query = rtrim($upd_query,", ");
        $upd_query .= " WHERE ".$where;
        return $upd_query;

    }
    
    /**
    * предварительно обрабатывает данные перед запросом. строковые данные экранируети оборачивает в кавычки
    * @param link array одномерный массив данных
    * 
    * @author Rolland
    */
    static function filterDataForQuery(&$data){
        foreach($data as $field=>&$value){
            if(is_string($value) && strpos($value,'compress')!==0 ){ // строковые данные оборачиваем в кавычки
                $value = "'".mysql_real_escape_string($value)."'";
            }
        }    
    }
    
    /**
    * отдает текущее время сервера в формате 2012-11-02 15:49:18 или 2012-11-02
    * @author Rolland
    */ 
    static function now_mysql($only_date=false){
       $pattern='Y-m-d';
       if(!$only_date)
          $pattern.=' H:i:s';
       return date($pattern);
    }
    
    
    /**
    * выполняет curl запрос на переданный адрес и отдает декодированные из json полученные данные  
    * в случае ошибки - отдает массив с ключом error и текстом ошибки. 
    * если нет ошибки, но декодировать не удается, отдает error = 'cannot decode' и оригинальный ответ в ключе encoded_res
    * @param string путь для запроса
    * @param array данные для POST
    * 
    * @return array
    */
    static function curl_exchange($url,$data=array(),$verify_host = false){
        $curl = curl_init($url);
        if(!empty($data)){   
            curl_setopt($curl, CURLOPT_POST, 1);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        }
        if(!empty($verify_host)){
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, FALSE);
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2);
        }
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        $s = curl_exec($curl);
        if(!$s){
            @curl_close($curl);
            return array(
                'error'=>curl_error($curl)
            );
        }
        @curl_close($curl);
        $res = @json_decode($s);    
        if($res) 
            return $res;
        return array(
            'error'=>'cannot decode',
            'encoded_res'=>$s
        );
    }
    
    /**
    * склеивает параметры в строку типа 'param=value&param2=value2' или 'param=valueparam2=value2'
    * служитдля формироваия подписанных урлов для одноклассников
    */
    static function join_params($params,$parts_div='&',$encode = true){
        foreach($params as $key=>$item){
            $params_joined .= ($params_joined?$parts_div:'').$key.'='.($encode?urlencode($item):$item);
        }
        return $params_joined;    
    }
    
}

$SPExport_listener = new SPExport();
 
